from django.conf.urls import url
from django.views.generic.base import RedirectView

from pigeon.tests.views import foo, foo_api


urlpatterns = [
    url(r'^api/foo/$', foo_api),
    url(r'^foo/$', foo),
    url(r'^bar/$', RedirectView.as_view(url='/foo/', permanent=False)),
]



from six.moves.urllib import parse


def strip_params_from_url(url):
    return url.split('?')[0]


def add_params_to_url(url, params):
    url_parts = list(parse.urlparse(url))
    query = dict(parse.parse_qsl(url_parts[4]))
    query.update(params)
    url_parts[4] = parse.urlencode(query)
    return parse.urlunparse(url_parts)



from django.http import HttpResponse, JsonResponse


def foo(request):
    return HttpResponse('Hello World!')


def foo_api(request):
    if request.method == 'GET':
        return JsonResponse({'foo': 'bar'})
    elif request.method == 'POST':
        return HttpResponse(status=204)



from pigeon.test import RenderTestCase


class RenderFooTestCase(RenderTestCase):

    def get200s(self):
        return [
            '/foo/',
        ]

    def testFooViewReturnsHelloWorld(self):
        response = self.assertResponseRenders('/foo/')
        self.assertIn(b'Hello World!', response.content)

    def testFooAPIView(self):
        self.assertAPIResponseRenders('/api/foo/')

    def testPostFooAPIViewReturns204(self):
        self.assertAPIResponseRenders('/api/foo/', status_code=204, method='POST', data={'text': 'Hello World!'})

    def testBarRedirectsToFoo(self):
        self.assertResponseRedirects('/bar/', '/foo/')



#!/usr/bin/env python
from setuptools import setup, find_packages, Command


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import django
        from django.conf import settings
        from django.core.management import call_command

        settings.configure(
            DATABASES={
                'default': {
                    'NAME': ':memory:',
                    'ENGINE': 'django.db.backends.sqlite3',
                }
            },
            INSTALLED_APPS=('pigeon',),
            ROOT_URLCONF='pigeon.tests.urls'
        )
        django.setup()
        call_command('test', 'pigeon')


setup(
    name='django-pigeon',
    version='0.1.1',
    packages=find_packages(),
    license='ISC License',
    description='Test utilities for Django projects.',
    url='https://github.com/RevolutionTech/django-pigeon/',
    author='Lucas Connors',
    author_email='lucas@revolutiontech.ca',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Testing',
        'Topic :: Utilities',
    ],
    install_requires=[
        'six',
        'Django >= 1.8',
    ],
    cmdclass={'test': TestCommand},
)



import json

from django.test import TestCase, TransactionTestCase

from pigeon.url.utils import strip_params_from_url, add_params_to_url


class RenderTestCaseMeta(type):

    def __new__(cls, name, bases, dct):
        def testRender200s(self):
            for url in self.get200s():
                self.assertResponseRenders(url)

        if 'get200s' in dct:
            dct['testRender200s'] = testRender200s
        return type.__new__(cls, name, bases, dct)


class RenderTestCaseMixin(object):

    __metaclass__ = RenderTestCaseMeta

    def assertResponseRenders(self, url, status_code=200, method='GET', data={}, has_form_error=False, **kwargs):
        request_method = getattr(self.client, method.lower())
        follow = status_code == 302
        response = request_method(url, data=data, follow=follow, **kwargs)

        if status_code == 302:
            redirect_url, response_status_code = response.redirect_chain[0]
        else:
            response_status_code = response.status_code
        self.assertEquals(
            response_status_code,
            status_code,
            "URL {url} returned with status code {actual_status} when {expected_status} was expected.".format(
                url=url,
                actual_status=response_status_code,
                expected_status=status_code
            )
        )

        # Check that forms submitted did not return errors (or did if it should have)
        form_error_assertion_method = self.assertIn if has_form_error else self.assertNotIn
        form_error_assertion_method(b'errorlist', response.content)

        return response

    def assertAPIResponseRenders(self, url, status_code=200, method='GET', data={}, **kwargs):
        api_url = add_params_to_url(url, {'format': 'json'})
        if data:
            data = json.dumps(data)
        response = self.assertResponseRenders(
            api_url,
            status_code=status_code,
            method=method,
            data=data,
            content_type='application/json',
            **kwargs
        )
        if status_code in [
            204,  # No Content
            205,  # Reset Content
        ]:
            return response
        return json.loads(response.content.decode())

    def assertResponseRedirects(self, url, redirect_url, status_code=200, method='GET', data={}, **kwargs):
        response = self.assertResponseRenders(url, status_code=302, method=method, data=data, **kwargs)
        redirect_url_from_response, _ = response.redirect_chain[0]
        relative_url_from_response = strip_params_from_url(redirect_url_from_response).replace('http://testserver', '')
        self.assertEquals(relative_url_from_response, redirect_url)
        self.assertEquals(response.status_code, status_code)


class RenderTestCase(RenderTestCaseMixin, TestCase):
    pass


class RenderTransactionTestCase(RenderTestCaseMixin, TransactionTestCase):
    pass
